#!/usr/bin/env python

# Author                 Version      Date
# -----------------------------------------------
# Martin Still (SAAO)    1.0          31 Aug 2006

# saltgain...

from pyraf import iraf 
import saltprint, saltkey, saltio
import time

# -----------------------------------------------------------
# core routine

def saltgain(images,gaindb,mult,logfile,verbose,status):

    status  = 0

# declare some lists

    infiles = []
    dbspeed = []
    dbrate  = []
    dbgain  = []
    dbnoise = []
    dbbias  = []
    dbamp   = []

# test the logfile

    logfile = saltio.logname(logfile)

# log the call 

    saltprint.line(logfile,verbose)
    history = 'SALTGAIN -- '
    history += 'images='+images+' '
    history1 = 'gaindb='+gaindb+' '
    yn = 'n'
    if (mult): yn = 'y'
    history1 += 'mult='+yn+' '
    history1 += 'logfile='+logfile+' '
    verbose = 'n'
    if (verbose): verbose = 'y'
    history1 += 'verbose='+verbose
    history  += history1
    saltprint.log(logfile,history,verbose)

# start time

    saltprint.time('\nSALTGAIN -- started at',logfile,verbose)

# is the input file specified?
   
    images = images.strip()
    if (status == 0): status = saltio.filedefined('Input',images,logfile)

# if the input file is a list, does it exist?

    if (status == 0 and images[0] == '@'): 
	status = saltio.listexists('Input',images,logfile)

# parse list of input files

    if (status == 0):
	infiles, status = saltio.listparse('Image',images,'',infiles,'',logfile)

# check raw input files exist

    if (status == 0): status = saltio.filesexist(infiles,'','r',logfile)

# check gain database file exists

    gaindb = gaindb.strip()
    if (status == 0): status = saltio.fileexists(gaindb,logfile)

# read gain database file

    if (status == 0):
	dbspeed,dbrate,dbgain,dbnoise,dbbias,dbamp,status = saltio.readgaindb(gaindb,logfile)

# open each raw image file

    for infile in infiles:
	if (status == 0): struct, status = saltio.openupdatefits(infile,logfile)

# gain correct the data

	if (status == 0):
	    history = 'SALTGAIN -- '
            history += 'images='+infile+' '
            history += history1
	    struct, gstatus = gain(struct,infile,mult,dbspeed,dbrate,dbgain,dbnoise,dbbias,dbamp,
				  gaindb,history,logfile,verbose,status)
	    if (gstatus == 1):
	       message = "SALTGAIN ERROR: "+infile+"not corrected"
	       saltprint.log(logfile,message,verbose)


# write FITS file

	if (status == 0): status = saltio.updatefits(struct,logfile)
	if (status == 0): status = saltio.closefits(struct,logfile)

# end time

    if (status == 0):
	saltprint.time('\nSALTGAIN -- completed at',logfile,verbose)
    else:
	saltprint.time('\nSALTGAIN -- aborted at',logfile,verbose)

# -----------------------------------------------------------
# gain correct data

def gain(struct,infile,mult,dbspeed,dbrate,dbgain,dbnoise,dbbias,dbamp,
	 gaindb,history,logfile,verbose,status):

# identify instrument

    if (status == 0): instrume,keyprep,keygain,keybias,keyxtalk,keyslot,status = \
	saltkey.instrumid(struct,infile,logfile)

# has file been prepared?

    if (status == 0): status = saltkey.prepare(struct,infile,keyprep,logfile)

# has file been gain corrected?

    if (status == 0): status = 2
    if (status == 2):
	try:
	    key = struct[0].header[keygain]
	except:
	    status = 0
    if (status == 2):
	message = 'ERROR -- SALTGAIN: file ' + infile + ' has already been gain-corrected'
	status = saltprint.err(logfile,message)

# how many extensions?

    if (status == 0): nextend,status = saltkey.get('NEXTEND',struct[0],infile,logfile)

# how many amplifiers?

    if (status == 0): 
	amplifiers,status = saltkey.get('NCCDS',struct[0],infile,logfile)
	amplifiers *= 2

# housekeeping keywords

    if (status == 0): status = saltkey.put('SAL-TLM',time.asctime(time.localtime()),
					   struct[0],infile,logfile)
    if (status == 0): status = saltkey.new(keygain,time.asctime(time.localtime()),
					   'Images have been gain corrected',struct[0],infile,logfile)
    if (status == 0): status = saltkey.history(struct[0],history,infile,logfile)


# read speed and gain of each exposure

    if (status == 0):
	gainset,status = saltkey.get('GAINSET',struct[0],infile,logfile)
	rospeed,status = saltkey.get('ROSPEED',struct[0],infile,logfile)

# check image file and gain database are compatible

    if (status == 0):
	ngains = len(dbgain)
	if (int(max(dbamp)) != amplifiers):
	    message  = '\nERROR -- SALTGAIN: ' + infile + ' contains ' + str(amplifiers) + ' amplifiers'
	    message += ', the gaindb file ' + gaindb + ' contains ' + str(max(dbamp)) + ' amplifiers'
	    status = saltprint.err(logfile,message)

	if gainset not in dbrate: 
	    message  = '\nERROR -- SALTGAIN: ' + infile + ' GAIN does not match any vaue in ' + gaindb
            status = saltprint.err(logfile,message)

	if rospeed not in dbspeed:
	    message  = '\nERROR -- SALTGAIN: ' + infile + ' ROSPEED does not match any vaue in ' + gaindb + ' '
            status = saltprint.err(logfile,message)


# write keyword to individual HDUs

    if (status == 0):
	message = '%28s %6s %5s %3s %5s %5s' \
	    % ('HDU','GAIN','SPEED','AMP','GAIN','NOISE')
	saltprint.log(logfile,'\n      ---------------------------------------------------',verbose)
	saltprint.log(logfile,message,verbose)
	saltprint.log(logfile,'      ---------------------------------------------------',verbose)
	for i in range(nextend):
	    hdu = i + 1
	    amplifier = hdu%amplifiers
	    if (amplifier == 0): amplifier = amplifiers
	    for j in range(ngains):
		if (gainset == dbrate[j] and rospeed == dbspeed[j] and
		    amplifier == int(dbamp[j])):
		    status = saltkey.put('GAIN',dbgain[j],struct[hdu],infile,logfile)
		    status = saltkey.put('RDNOISE',dbnoise[j],struct[hdu],infile,logfile)
		    if (mult): 
			gainmult = 1.
		    else:
			gainmult = dbgain[j]
		    status = saltkey.new('GAINMULT',gainmult,'Gain multiplication',
					 struct[hdu],infile,logfile)

# read image
		    if (status == 0):
			imagedata = struct[hdu].data
			naxis1,status = saltkey.get('NAXIS1',struct[hdu],infile,logfile)
			naxis2,status = saltkey.get('NAXIS2',struct[hdu],infile,logfile)
			
# multiply images by gains

		    if (status == 0 and mult):
			try:
			    imagedata[0:naxis2-1,0:naxis1-1] *= float(dbgain[j])
			except:
			    mesage = 'ERROR -- SALTGAIN: Cannot perform arithmetic on image '
			    message += infile+'['+str(hdu)+']'
			    status = saltprint.err(logfile,message)

# log operation

		    if (status ==  0):
			infilename = infile.split('/')
			infilename = infilename[len(infilename)-1]
			message = '%25s[%1d] %6s %5s %2s %6.2f %5.2f' \
			    % (infilename,hdu,gainset,rospeed,amplifier,
			       float(dbgain[j]),float(dbnoise[j]))
			saltprint.log(logfile,message,verbose)
	
	if (status == 0): 
	   saltprint.time('\nSALTGAIN -- updated ' + infilename + ' at',logfile,verbose)
	else:
	   message = 'ERROR -- SALTGAIN not updated'
           message += infile+'['+str(hdu)+']'
           status = saltprint.err(logfile,message)


    return struct, status

# -----------------------------------------------------------
# main code 

parfile = iraf.osfn("saltred$saltgain.par") 
t = iraf.IrafTaskFactory(taskname="saltgain",value=parfile,function=saltgain, pkgname='saltred')
