#!/usr/bin/env python

# Author                 Version      Date
# -----------------------------------------------
# Martin Still (SAAO)    1.0          16 Oct 2006

# saltxtalk...

from pyraf import iraf 

import saltprint, saltkey, saltio
import os, time

# -----------------------------------------------------------
# core routine

def saltxtalk(images,outimages,outpref,xtalkfile,clobber,logfile,verbose,status):

    status = 0
    ifil = 0
    xcoeff = []

# test the logfile

    logfile = saltio.logname(logfile)

# log the call 

    saltprint.line(logfile,verbose)
    history = 'SALTXTALK -- '
    history += 'images='+images+' '
    history += 'outimages='+outimages+' '
    history1 = 'outpref='+outpref+' '
    history1 += 'xtalkfile='+xtalkfile+' '
    yn = 'n'
    if (clobber): yn = 'y'+' '
    history1 += 'clobber='+yn
    history1 += 'logfile='+logfile+' '
    yn = 'n'
    if (verbose): yn = 'y'
    history1 += 'verbose='+yn
    history += history1
    saltprint.log(logfile,history,verbose)

# start time

    saltprint.time('\nSALTXTALK -- started at',logfile,verbose)

# is the input file specified?
   
    if (status == 0): status = saltio.filedefined('Input',images,logfile)

# if the input file is a list, does it exist?

    if (status == 0 and images[0] == '@'): 
        status = saltio.listexists('Input',images,logfile)

# parse list of input files

    if (status == 0):
        infiles, status = saltio.listparse('Raw image',images,'','','',logfile)

# check input files exist

    if (status == 0): status = saltio.filesexist(infiles,'','r',logfile)

# load output name list: @list, * and comma separated
   
    outimages = outimages.strip()
    outpref = outpref.strip()
    if (status == 0 and len(outpref) == 0 and len(outimages) == 0):
        message = '\nERROR -- SALTXTALK: output file(s) not specified'
        status = saltprint.err(logfile,message)

# test output @filelist exists

    if (status == 0 and len(outimages) > 0 and outimages[0] == '@'): 
        status = saltio.listexists('Output',outimages,logfile)

# parse list of output files

    if (status == 0):
        outfiles, status = saltio.listparse('Output image',outimages,outpref,infiles,'',logfile)

# are input and output lists the same length?

    if (status == 0): status = saltio.comparelists(infiles,outfiles,'Input','output',logfile)

# do the output files already exist?

    if (status == 0 and not clobber): status = saltio.filesexist(outfiles,'','w',logfile)

# does crosstalk coefficient data exist

    xtalkfile = xtalkfile.strip()
    if (status == 0): status = saltio.fileexists(xtalkfile,logfile)

# read crosstalk coefficent file

    if (status == 0): xcoeff, status = saltio.readxtalkcoeff(xtalkfile,logfile,status)

# open each raw image file

    for infile in infiles:
	outfile = outfiles[ifil]
	if (status == 0): struct, status = saltio.openfits(infile,logfile)

# crosstalk correction

	if (status == 0): 
	      history = 'SALTXTALK -- '
              history += 'images='+infile+' '
              history += 'outimages='+outfile+' '
	      history += history1
              struct,status = xtalk(struct,infile,outfile,xtalkfile,
						xcoeff,history,logfile,verbose)

# clobber old files
	    
	if (status == 0 and clobber and os.path.isfile(outfile)): 
	    status = saltio.delete(outfile,False,logfile)

# write FITS file		

	filename = outfile.split('/')[-1]
	if (status == 0): status = saltio.writefits(struct,outfile,logfile)
	if (status == 0): status = saltio.closefits(struct,logfile)
	if (status == 0): saltprint.time('\nSALTXTALK -- created ' + filename + ' at',logfile,verbose)
	ifil += 1

# end time

    if (status == 0):
	saltprint.time('\nSALTXTALK -- completed at',logfile,verbose)
    else:
	saltprint.time('\nSALTXTALK -- aborted at',logfile,verbose)

# -----------------------------------------------------------
# crosstalk correction

def xtalk(struct,infile,outfile,xtalkfile,xcoeff,history,logfile,verbose):

# identify instrument

    instrume,keyprep,keygain,keybias,keyxtalk,keyslot,status = saltkey.instrumid(struct,infile,logfile)

# has file been prepared?

    if (status == 0): status = saltkey.prepare(struct,infile,keyprep,logfile)

# has file already been corrected?

    if (status == 0): 
	status = 2
	try:
	    value = struct[0].header[keyxtalk]
	except:
	    status = 0
    if (status == 2): 
	message = 'ERROR -- SALTXTALK: ' + infile + ' has already been crosstalk corrected\n'
	status = saltprint.err(logfile,message)

# how many extensions?

    if (status == 0): nextend,status = saltkey.get('NEXTEND',struct[0],infile,logfile)

# how many amplifiers?

    if (status == 0): 
	amplifiers,status = saltkey.get('NCCDS',struct[0],infile,logfile)
	amplifiers *= 2

# check image file and cross talk database are compatible

    if (status == 0):
	if (len(xcoeff)-1 != amplifiers):
	    message  = '\nERROR -- SALTXTALK: ' + infile + ' contains ' + str(amplifiers) + ' amplifiers'
	    message += ', the cross talk file ' + xtalkfile + ' contains ' + str(len(xcoeff)-1) + ' amplifiers'
	    status = saltprint.err(logfile,message)

# housekeeping keywords

    if (status == 0):
	status = saltkey.put('SAL-TLM',time.asctime(time.localtime()),struct[0],outfile,logfile)
	status = saltkey.new(keyxtalk,time.asctime(time.localtime()),
			     'Images have been crosstalk corrected',struct[0],outfile,logfile)
    if (status == 0): status = saltkey.history(struct[0],history,outfile,logfile)

# log crosstalk/extension table

    if (status == 0):
	message = '%28s %23s' % ('HDU','Correction')
	saltprint.log(logfile,'\n      ----------------------------------------------',verbose)
	saltprint.log(logfile,message,verbose)
	saltprint.log(logfile,'      ----------------------------------------------',verbose)

# loop over image extensions

    if (status == 0):
	for i in range(nextend/2):
	    hdu = i * 2 + 1
	    amplifier = hdu%amplifiers
	    if (amplifier == 0): amplifier = amplifiers
	    if (status == 0 and hdu == 1):
		value,status = saltkey.get('NAXIS1',struct[hdu],infile,logfile)
		dx1 = int(value)
		value,status = saltkey.get('NAXIS1',struct[hdu+1],infile,logfile)
		dx2 = int(value)
	    if (status == 0):
		imagedata1,status = saltio.readimage(struct,hdu,logfile)
		imagedata2,status = saltio.readimage(struct,hdu+1,logfile)
		revimage1 = imagedata1 * float(xcoeff[amplifier])
		revimage2 = imagedata2 * float(xcoeff[amplifier+1])
		if (dx1 != dx2):
		    message = 'ERROR -- SALTXTALK: HDUs '+infile
		    message += '['+str(hdu)+'] and '+infile+'['+str(hdu+1)+']'
		    message += ' have different dimensions'
		    status = saltprint.err(logfile,message)
			    
# crosstalk correction
			
	    if (status == 0):
		for j in range(dx1):
		    imagedata1[:,j] -= revimage2[:,dx2-j-1]
		    imagedata2[:,j] -= revimage1[:,dx2-j-1]

# log crosstalk expression

	    if (status == 0):
		infilename = infile.split('/')
		infilename = infilename[len(infilename)-1]
		message = '%25s[%1d]  Amp%1d - Amp%1d * %8.6f' % \
		    (infilename, hdu, amplifier, amplifier+1,float(xcoeff[amplifier+1]))
		saltprint.log(logfile,message,verbose)
		message = '%25s[%1d]  Amp%1d - Amp%1d * %8.6f' % \
		    (infilename, hdu+1, amplifier+1, amplifier,float(xcoeff[amplifier]))
		saltprint.log(logfile,message,verbose)

# update image in HDU structure

	    if (status == 0): 
		struct,status = saltio.writeimage(struct,hdu,imagedata1,logfile)
		struct,status = saltio.writeimage(struct,hdu+1,imagedata2,logfile)

    return struct, status

# -----------------------------------------------------------
# main code 

parfile = iraf.osfn("saltred$saltxtalk.par") 
t = iraf.IrafTaskFactory(taskname="saltxtalk",value=parfile,function=saltxtalk, pkgname='saltred')
